<?php

namespace App\Factory;

use \Exception;

class OrderFactory
{
    private $connection;

    public function __construct(\Doctrine\DBAL\Connection $connection)
    {
        $this->connection = $connection;
    }

    public function getTable($nrMesa)
    {
        $nrMesa = str_pad($nrMesa, 4, 0, STR_PAD_LEFT);
        $sql = "
            SELECT CDFILIAL, CDLOJA, NRMESA, 
                   IDSTMESA, IDSTMESAAUX
                FROM MESA
                    WHERE NRMESA = :NRMESA 
                        AND IDSTMESA = 'A'
        ";

        $params = [
            ':NRMESA' => $nrMesa
        ];
        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getComanda($nrComanda)
    {
        $nrComanda = str_pad($nrComanda, 10, 0, STR_PAD_LEFT);
        $sql = "
            SELECT V.CDFILIAL, V.NRVENDAREST, V.NRMESA, V.CDVENDEDOR, V.CDOPERADOR,
                   V.NRPOSICAOMESA, CV.NRCOMANDA, CV.DSCOMANDA, CV.IDSTCOMANDA, 
                   CV.IDORGCMDVENDA, CV.NRPEDMOBILE, CV.NRORG, CV.VRCOMISVENDE  
            FROM COMANDAVEN CV
                JOIN VENDAREST V ON V.NRVENDAREST = CV.NRVENDAREST
              WHERE CV.DSCOMANDA = :DSCOMANDA
        ";

        $params = [
            ':DSCOMANDA' => $nrComanda
        ];

        $result = $this->connection->fetchAssoc($sql, $params);

        return $result;
    }

    public function getTableIsOpen($nrMesa)
    {
        $nrMesa = str_pad($nrMesa, 4, 0, STR_PAD_LEFT);
        $sql = "
            SELECT V.CDFILIAL, V.NRVENDAREST, V.NRMESA, V.CDVENDEDOR, V.CDOPERADOR,
                   V.NRPOSICAOMESA, CV.NRCOMANDA, CV.DSCOMANDA, CV.IDSTCOMANDA, 
                   CV.IDORGCMDVENDA, CV.NRPEDMOBILE, CV.NRORG  
            FROM VENDAREST V
                JOIN COMANDAVEN CV ON V.NRVENDAREST = CV.NRVENDAREST
              WHERE V.NRMESA = :NRMESA

        ";

        $params = [
            ':NRMESA' => $nrMesa
        ];

        $result = $this->connection->fetchAssoc($sql, $params);

        return $result;
    }

    public function sqlNewCode($cdContador)
    {
        $sql = "
            SELECT CDCONTADOR, NRSEQUENCIAL
                FROM NOVOCODIGO
                    WHERE CDCONTADOR = :CDCONTADOR
        ";

        $params = [
            ':CDCONTADOR' => $cdContador
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function insertVendarest($cdFilial, $cdLoja, $generalParams, $vendarestSeq, $nrMesa)
    {
        $sql = "
            INSERT INTO VENDAREST (
                CDFILIAL, NRVENDAREST, CDLOJA, NRMESA, CDVENDEDOR,
                DTHRABERMESA, DTHRFECHMESA, CDOPERADOR, NRPESMESAVEN,
                CDCLIENTE, CDCONSUMIDOR, IDPEDIDOPAGO
            )
            VALUES (
                :CDFILIAL, :NRVENDAREST, :CDLOJA, :NRMESA,
                :CDVENDEDOR, GETDATE(), null, :CDOPERADOR,
                :NRPESMESAVEN, :CDCLIENTE, :CDCONSUMIDOR, :IDPEDIDOPAGO
            )
        ";

        $params = [
            'CDFILIAL'     => $cdFilial,
            'NRVENDAREST'  => $vendarestSeq,
            'CDLOJA'       => $cdLoja,
            'NRMESA'       => !empty($nrMesa) ? $nrMesa : $generalParams['NRMESA'],
            'CDVENDEDOR'   => $generalParams['CDVENDEDOR'],
            'CDOPERADOR'   => $generalParams['CDOPERADOR'],
            'NRPESMESAVEN' => '1',
            'CDCLIENTE'    => $generalParams['CDCLIENTE'],
            'CDCONSUMIDOR' => null,
            'IDPEDIDOPAGO' => 'S'
        ];

        return $this->connection->executeQuery($sql, $params);
    }

    public function insertComandaven($cdFilial, $cdLoja, $generalParams, $vendarestSeq, $comandavenSeq, $dscomanda, $orderType, $IDSTCOMANDA)
    {
        $sql = "
            INSERT INTO COMANDAVEN (
                CDFILIAL, NRVENDAREST, NRCOMANDA, CDLOJA,
                DSCOMANDA, IDSTCOMANDA, SGSEXOCON, TXMOTIVCANCE,
                VRACRCOMANDA, CDPAIS, SGESTADO, CDMUNICIPIO,
                NRCEPCONSCOMAND, CDBAIRRO, DSBAIRRO, DSENDECONSCOMAN,
                DSCOMPLENDCOCOM, DSREFENDCONSCOM, IDORGCMDVENDA,
                IDRETBALLOJA, NRCOMANDAEXT, IDSINCAGENDA, DTHRAGENDADA, IDIMPCUPEXP,
                DSCONSUMIDOR, DSOBSCOMANDA, IDFECHAMENTOAPI
            )
            VALUES (
                :CDFILIAL, :NRVENDAREST, :NRCOMANDA, :CDLOJA,
                :DSCOMANDA, :IDSTCOMANDA, :SGSEXOCON, :TXMOTIVCANCE,
                :VRACRCOMANDA, :CDPAIS, :SGESTADO, :CDMUNICIPIO,
                :NRCEPCONSCOMAND, :CDBAIRRO, :DSBAIRRO, :DSENDECONSCOMAN,
                :DSCOMPLENDCOCOM, :DSREFENDCONSCOM, :IDORGCMDVENDA,
                :IDRETBALLOJA, :NRCOMANDAEXT, :IDSINCAGENDA, CONVERT(DATETIME, :DTHRAGENDADA, 103),
                :IDIMPCUPEXP, :DSCONSUMIDOR, :DSOBSCOMANDA, :IDFECHAMENTOAPI
            )
        ";

        $params = [
            'CDFILIAL'        => $cdFilial,
            'NRVENDAREST'     => $vendarestSeq,
            'NRCOMANDA'       => $comandavenSeq,
            'CDLOJA'          => $cdLoja,
            'IDSTCOMANDA'     => $IDSTCOMANDA,
            'DSCOMANDA'       => $dscomanda,
            'SGSEXOCON'       => 'M',
            'TXMOTIVCANCE'    => " ",
            'VRACRCOMANDA'    => 0,
            'CDPAIS'          => null,
            'SGESTADO'        => null,
            'CDMUNICIPIO'     => null,
            'NRCEPCONSCOMAND' => null,
            'CDBAIRRO'        => null,
            'DSBAIRRO'        => null,
            'DSENDECONSCOMAN' => null,
            'DSCOMPLENDCOCOM' => null,
            'DSREFENDCONSCOM' => null,
            'IDORGCMDVENDA'   => $orderType,
            'IDRETBALLOJA'    => 'N',
            'NRCOMANDAEXT'    => null,
            'IDSINCAGENDA'    => 'S',
            'DTHRAGENDADA'    => null,
            'IDIMPCUPEXP'     => '0',
            'DSCONSUMIDOR'    => null,
            'VRDESCFID'       => 0,
            'DSOBSCOMANDA'    => null,
            'IDFECHAMENTOAPI' => 'N'
        ];

        return $this->connection->executeQuery($sql, $params);
    }

    public function tableStatus($cdFilial, $nrMesa, $cdLoja)
    {
        $sql = "
            SELECT TOP 1 IDSTMESAAUX 
                FROM MESA
                    WHERE CDFILIAL    = :CDFILIAL
                        AND CDLOJA      = :CDLOJA
                        AND NRMESA IN (SELECT DISTINCT NRMESA
                                        FROM MESA
                                        WHERE CDFILIAL = :CDFILIAL
                                            AND CDLOJA   = :CDLOJA
                                            AND NRMESA   = :NRMESA
                                        UNION
                                        SELECT NRMESA
                                        FROM JUNCAOMESA JM, MESAJUNCAO MJ
                                        WHERE JM.CDFILIAL  = :CDFILIAL
                                            AND JM.CDFILIAL  = MJ.CDFILIAL
                                            AND JM.CDLOJA    = :CDLOJA
                                            AND JM.NRJUNMESA = MJ.NRJUNMESA
                                            AND JM.CDLOJA    = MJ.CDLOJA
                                            AND JM.NRJUNMESA = (SELECT NRJUNMESA
                                                                FROM MESAJUNCAO
                                                                WHERE CDFILIAL = :CDFILIAL
                                                                AND CDLOJA   = :CDLOJA
                                                                AND NRMESA   = :NRMESA))
        ";

        $params = [
            ':CDFILIAL' => $cdFilial,
            ':CDLOJA'   => $cdLoja,
            ':NRMESA'   => $nrMesa
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function updateStatusTable($nrMesa)
    {
        $sql = "
            UPDATE MESA 
                SET IDSTMESAAUX = 'O' 
                    WHERE NRMESA = :NRMESA
        ";

        $params = [
            ':NRMESA' =>  $nrMesa
        ];

        return $this->connection->executeQuery($sql, $params);
    }

    public function updateTableOrder($nrMesa, $nrVendarest)
    {
        $sql = "
            UPDATE VENDAREST 
                SET NRMESA = :NRMESA
                    WHERE NRVENDAREST = :NRVENDAREST
        ";

        $params = [
            ':NRMESA' =>  $nrMesa,
            ':NRVENDAREST' => $nrVendarest
        ];

        return $this->connection->executeQuery($sql, $params);
    }

    public function getIdcontroprod($cdLoja, $cdFilial)
    {
        $sql = "
            SELECT IDCONTROPROD
                FROM LOJA
                    WHERE CDFILIAL = :CDFILIAL
                     AND CDLOJA = :CDLOJA
        ";

        $params = [
            ':CDFILIAL' => $cdFilial,
            ':CDLOJA' => $cdLoja
        ];
        $result = $this->connection->fetchAll($sql, $params);
        $idcontroprod = null;
        if (isset($result[0])) {
            $idcontroprod = $result[0]['IDCONTROPROD'];
        }
        return $idcontroprod;
    }

    public function getOrderData($nrVendarest, $nrcomanda, $nrProdComVen)
    {
        $sql = "
            SELECT IC.CDFILIAL, IC.NRVENDAREST, IC.NRCOMANDA, IC.NRPRODCOMVEN, IC.CDPRODUTO, P.NMPRODUTO,
               IC.QTPRODCOMVEN, IC.TXPRODCOMVEN, IC.NRSEQPRODCOM, IC.CDPRODPROMOCAO, P.IDIMPRODUVEZ,
               P.IDPESAPROD, IC.IDOBSERVACAOAD
            FROM ITCOMANDAVEN IC JOIN PRODUTO P ON IC.CDPRODUTO = P.CDPRODUTO
                WHERE IC.NRVENDAREST = :NRVENDAREST
                AND IC.NRCOMANDA = :NRCOMANDA
                AND IC.NRPRODCOMVEN IN (:ARR)
        ";

        $params = [
            ':NRVENDAREST' => $nrVendarest,
            ':NRCOMANDA'   => $nrcomanda,
            ':ARR'         => $nrProdComVen
        ];

        $type = array(
            'ARR' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY
        );
        return $this->connection->fetchAll($sql, $params, $type);
    }

    public function setServiceTax($cmdDetails, $serviceTax)
    {
        $sql = "
            UPDATE COMANDAVEN
                SET 
                    VRCOMISVENDE = :VRCOMISVENDE
                WHERE CDFILIAL = :CDFILIAL
                 AND NRVENDAREST = :NRVENDAREST
                 AND NRCOMANDA = :NRCOMANDA
        ";

        $params = [
            ':VRCOMISVENDE' => !empty($serviceTax) ? $serviceTax : 0.001,
            ':CDFILIAL'     => $cmdDetails['CDFILIAL'],
            ':NRVENDAREST'  => $cmdDetails['NRVENDAREST'],
            ':NRCOMANDA'    => $cmdDetails['NRCOMANDA']
        ];

        return $this->connection->executeQuery($sql, $params);
    }

    public function getCanceledProducts()
    {
        $sql = "
            SELECT IV.CDFILIAL, IV.NRVENDAREST, IV.NRCOMANDA, CV.DSCOMANDA, IV.CDPRODUTO,
                    P.NMPRODUTO, IV.QTPRODCOMVEN, IV.VRPRECCOMVEN, IV.CDSENHAPED,IV.DTHRPRODCANVEN,
                   IV.NRPRODCOMVEN, IV.IDPRODEXT, VR.NRMESA, CV.IDORGCMDVENDA
                FROM ITCOMANDAVEN IV
                    JOIN COMANDAVEN CV 
                        ON IV.NRCOMANDA = CV.NRCOMANDA 
                        AND IV.NRVENDAREST = CV.NRVENDAREST
                    JOIN PRODUTO P 
                        ON IV.CDPRODUTO = P.CDPRODUTO
                    JOIN VENDAREST VR
                        ON VR.NRVENDAREST = IV.NRVENDAREST
                    WHERE IDSTPRCOMVEN = '6'
                        AND IV.IDPRODEXT IS NOT NULL
                        AND (DTHRPRODCANVEN BETWEEN DATEADD(MINUTE, -5, GETDATE()) AND GETDATE())
                        AND (CV.IDORGCMDVENDA LIKE 'MES%' OR  CV.IDORGCMDVENDA LIKE 'CMD%')
        ";

        return $this->connection->fetchAll($sql);
    }

    public function getUnsyncedProducts()
    {
        $sql = "
            SELECT IV.CDFILIAL, IV.NRVENDAREST, IV.NRCOMANDA, CV.DSCOMANDA, IV.CDPRODUTO,
                    P.NMPRODUTO, IV.QTPRODCOMVEN, IV.VRPRECCOMVEN, IV.CDSENHAPED,IV.DTHRPRODCANVEN,
                   IV.NRPRODCOMVEN, IV.IDPRODEXT, VR.NRMESA, IV. VRDESCCOMVEN, IV.VRACRCOMVEN, VR.NRMESA, CV.IDORGCMDVENDA
            FROM ITCOMANDAVEN IV
                JOIN COMANDAVEN CV 
                    ON IV.NRCOMANDA = CV.NRCOMANDA 
                    AND IV.NRVENDAREST = CV.NRVENDAREST
                JOIN PRODUTO P 
                    ON IV.CDPRODUTO = P.CDPRODUTO
                JOIN VENDAREST VR
                    ON VR.NRVENDAREST = IV.NRVENDAREST
                WHERE IV.IDPRODEXT IS NULL
                    AND IV. IDSTPRCOMVEN <> '6'
                    AND (CV.IDORGCMDVENDA LIKE 'MES%' OR  CV.IDORGCMDVENDA LIKE 'CMD%')
        ";

        return $this->connection->fetchAll($sql);
    }

    public function updateComandaVen($productDetails, $idproext)
    {
        $sql = "
            UPDATE ITCOMANDAVEN
              SET IDPRODEXT = :IDPRODEXT
                WHERE CDFILIAL       = :CDFILIAL
                    AND NRVENDAREST  = :NRVENDAREST
                    AND NRCOMANDA    = :NRCOMANDA
                    AND NRPRODCOMVEN = :NRPRODCOMVEN
        ";

        $params = [
            ':CDFILIAL'     => $productDetails['CDFILIAL'],
            ':NRVENDAREST'  => $productDetails['NRVENDAREST'],
            ':NRCOMANDA'    => $productDetails['NRCOMANDA'],
            ':NRPRODCOMVEN' => $productDetails['NRPRODCOMVEN'],
            ':IDPRODEXT'    => $idproext,
        ];

        return $this->connection->executeQuery($sql, $params);
    }

    public function getSales()
    {
        $sql = "
            SELECT NRSEQVENDA, NRVENDAREST, DSCOMANDAVND, NRMESA, IDORIGEMVENDA, VRTXSEVENDA
                FROM VENDA
                    WHERE (DTVENDA BETWEEN DATEADD(MINUTE, -5, GETDATE()) AND GETDATE())
                        AND (IDORIGEMVENDA LIKE 'MES%' OR  IDORIGEMVENDA LIKE 'CMD%')
                        AND IDSITUVENDA = 'O'
                        AND NRVENDAREST NOT IN (SELECT NRVENDAREST FROM COMANDAVEN
									             WHERE IDSTCOMANDA NOT IN  ('X', 'P'))
                        ORDER BY DTVENDA
        ";

        return $this->connection->fetchAll($sql);
    }

    public function getTransferProduct()
    {
        $sql = "
        SELECT IV.CDFILIAL, IV.NRVENDAREST, IV.NRCOMANDA, CV.DSCOMANDA, IV.CDPRODUTO,
               P.NMPRODUTO, IV.QTPRODCOMVEN, IV.VRPRECCOMVEN, IV.CDSENHAPED,IV.DTHRPRODCANVEN,
               IV.NRPRODCOMVEN, IV.IDPRODEXT, VR.NRMESA, CV.IDORGCMDVENDA, IV.NRMESADSCOMORIT
            FROM ITCOMANDAVEN IV
                JOIN COMANDAVEN CV 
                    ON IV.NRCOMANDA = CV.NRCOMANDA 
                    AND IV.NRVENDAREST = CV.NRVENDAREST
                JOIN PRODUTO P 
                    ON IV.CDPRODUTO = P.CDPRODUTO
                JOIN VENDAREST VR
                    ON VR.NRVENDAREST = IV.NRVENDAREST
                WHERE IDSTPRCOMVEN <> '6'
                    AND IV.IDPRODEXT IS NOT NULL
                    AND NRMESADSCOMORIT IS NOT NULL
                    AND (CV.IDORGCMDVENDA LIKE 'MES%' OR  CV.IDORGCMDVENDA LIKE 'CMD%')
                    AND (DTHRPRODCANVEN BETWEEN DATEADD(MINUTE, -5, GETDATE()) AND GETDATE())
        ";

        return $this->connection->fetchAll($sql);
    }

    public function getComandaEst($cdFilial, $nrVendaRest, $nrComanda, $nrProdComVen) {
        $sql = "
            SELECT  P.NMPRODUTO, P.IDIMPRODUVEZ, P.IDPESAPROD, IC.* 
                FROM ITCOMANDAEST IC
                    JOIN PRODUTO P ON  P.CDPRODUTO = IC.CDPRODUTO
                WHERE CDFILIAL = :CDFILIAL
                    AND NRVENDAREST = :NRVENDAREST
                    AND NRCOMANDA = :NRCOMANDA
                    AND NRPRODCOMVEN = :NRPRODCOMVEN
        ";

        $params = [
            ':CDFILIAL'     => $cdFilial,
            ':NRVENDAREST'  => $nrVendaRest,
            ':NRCOMANDA'    => $nrComanda,
            ':NRPRODCOMVEN' => $nrProdComVen
        ];
        return $this->connection->fetchAll($sql, $params);
    }
}

