<?php

namespace App\Service;

use App\Exception;

class ProductService
{

    private $productFactory;
    private $orderService;
    private $util;

    public function __construct(
        \App\Factory\ProductFactory $productFactory,
        \App\Service\OrderService $orderService,
        \App\Util\Util $util
    ) {
        $this->productFactory = $productFactory;
        $this->orderService = $orderService;
        $this->util = $util;
    }

    public function processProducts($items, $orderType, $cdFilial, $nrVendarest, $nrComanda, $cdLoja, $idControProd, $sellerPos, $cdvendedor, $pagerId)
    {   
        $products['ITCOMANDAVEN'] = [];
        $products['ITCOMANDAEST'] = [];
        $arrRodizio = [];

        foreach($items as $item) {
            if(isset($item['price']) && $item['price'] >= 0.01) {
                $simpleProducts = $this->simpleProducts($item, $orderType, $cdFilial, $nrVendarest, $nrComanda, $cdLoja, $idControProd, $sellerPos, $cdvendedor, $pagerId);
                foreach($simpleProducts as $simpleProd) {
                  array_push($products['ITCOMANDAVEN'], $simpleProd);
                }
                
            } else if((isset($item['price']) && $item['price'] == 0) && (isset($item['main_item_id']) && !empty($item['main_item_id']))) {
                array_push($arrRodizio, $item);
            } else {
                throw new \Exception("Não foi possível identificar qual a forma de processamento dos produtos.");
            }
        }
        foreach($arrRodizio as $prod) {
          if(!empty($products['ITCOMANDAVEN'])) {
            foreach($products['ITCOMANDAVEN'] as $prodPrincipal) {
                if($prodPrincipal['IDPRODEXT'] === $prod['main_item_id'])
                    $prodRodizio = $prodPrincipal;
            }
          }
  
          $product = $this->processRodizio($prod, $orderType, $cdFilial, $nrVendarest, $nrComanda, $cdLoja, $idControProd, $sellerPos, $cdvendedor, $pagerId, $prodRodizio);
          array_push($products['ITCOMANDAEST'], $product[0]);
        }

        return $products;
    }

    private function simpleProducts($item, $orderType, $cdFilial, $nrVendarest, $nrComanda, $cdLoja, $idControProd, $sellerPos, $cdvendedor, $pagerId)
    {
        $orderItemsSeq = $this->util->geraCodigo("ITCOMANDAVEN$cdFilial", 1, 10);
        $NRPRODCOMVEN  = ($this->util->geraCodigo("ITCOMANDAVEN{$cdFilial}{$nrComanda}", 1, 6) - 1);
        $NRSEQPRODCOM  = $NRPRODCOMVEN;

        $simpleItens = [];

        if (empty($item['price']) || ($item["price"] < 0.01) && !isset($item["main_item_id"]))
            throw new \Exception("O produto: {$item['code']} - {$item['name']} não atende o preço mínimo por item (R$ 0,01).");

        if (empty($item['code']))
            throw new \Exception("O produto: {$item['code']} - {$item['name']} não possui código de integração parametrizado.");

        $productDetails = $this->productFactory->getProduct($item['code']);
        if (empty($productDetails))
            throw new \Exception("O produto: {$item['code']} - {$item['name']} não encontrado. Verifique a parametrização.");


        $productDetails = array_merge($productDetails, $item);

        $productDetails["TXPRODCOMVEN"] = " ";
        $hasComplement = false;

        if (!empty($item["options"]) && count($item["options"]) > 0) {
            foreach ($productDetails["options"] as $obs) {
                if (strlen($obs['code']) === 5 && substr($obs['code'], 2, 1) === '|') {
                    $obsTitle = $this->getObservationName($obs);
                    for ($i = 0; $i < $obs["quantity"]; $i++)
                        $productDetails["TXPRODCOMVEN"] .= $obsTitle . "; ";
                }
                if (($obs['price'] > 0.01) && (!$hasComplement))
                    $hasComplement = true;
            }
        }
        $CDPRODPROMO = null;
        if ((!empty($item["options"])) && ($hasComplement))
            $CDPRODPROMO = $productDetails["CDPRODUTO"];

        $notes = "";

        if (!empty($item['notes']) && count($item['notes']) > 0) {
            foreach ($item['notes'] as $note) {
                $notes .= $note . "; ";
            }
        }

        $productDetails["TXPRODCOMVEN"] .= empty($productDetails["notes"]) ? "" : implode(",", $productDetails["notes"]) . "; ";

        $NRPRODCOMVEN = ($this->util->geraCodigo("ITCOMANDAVEN{$cdFilial}{$nrComanda}", 1, 6) - 1);
        $NRSEQPRODCOM = $NRPRODCOMVEN;
        $productReturn = array(
            "CDFILIAL"          => $cdFilial,
            "NRVENDAREST"       => $nrVendarest,
            "NRCOMANDA"         => $nrComanda,
            "NRPRODCOMVEN"      => $this->util->toRight($NRPRODCOMVEN, 6),
            "CDPRODUTO"         => $productDetails["CDPRODUTO"],
            "QTPRODCOMVEN"      => $item['quantity'],
            "VRPRECCOMVEN"      => $this->util->toFloat($item['price']),
            "TXPRODCOMVEN"      => $productDetails["TXPRODCOMVEN"],
            "IDSTPRCOMVEN"      => $idControProd === "S" ? "1" : "3",
            "VRDESCCOMVEN"      => 0.00,
            "NRLUGARMESA"       => "01",
            "IDPRODIMPFIS"      => "N",
            "CDLOJA"            => $cdLoja,
            "VRACRCOMVEN"       => 0.00,
            "NRSEQPRODCOM"      => $this->formatNrSeqProdCom($NRPRODCOMVEN),
            "NRSEQPRODCUP"      => $orderItemsSeq,
            "VRPRECCLCOMVEN"    => 0.00,
            "CDCAIXACOLETOR"    => $sellerPos,
            "CDPRODPROMOCAO"    => null,
            "CDVENDEDOR"        => $cdvendedor,
            "NRINSCRCONS"       => null,
            "DSOBSPEDDIGCMD"    => $notes,
            "CDSENHAPED"        => $pagerId,
            "ITEM_OBSERVATION"  => [],
            "IDOBSERVACAOAD"    => 'N',
            "IDPRODEXT"         => $item['id']
        );

        if (!empty($productDetails["options"]) && count($productDetails["options"]) > 0) {
            foreach ($productDetails["options"] as $obs) {
                if ($obs['price'] >= 0.01) {
                    $isCombo = false;
                    if (strlen($obs['code']) === 5 && substr($obs['code'], 2, 1) === '|') {
                        $observationDetails = $this->productFactory->getObservation($obs['code'], true);
                    } else {
                        $isCombo = true;
                        $observationDetails = $this->productFactory->getProduct($obs['code']);
                    }

                    if (empty($observationDetails))
                        throw new \Exception("Não foi possivel encontrar o complemento de código: {$obs['code']}");

                    for ($y = 0; $y < $obs['quantity']; $y++) {
                        $NRPRODCOMVEN = ($this->util->geraCodigo("ITCOMANDAVEN{$cdFilial}{$nrComanda}", 1, 6) - 1);
                        $observationDetails["NRPRODCOMVEN"] = str_pad((string) ($NRPRODCOMVEN), 6, '0', STR_PAD_LEFT);


                        $obsReturn = array(
                            "CDFILIAL"       => $cdFilial,
                            "NRVENDAREST"    => $nrVendarest,
                            "NRCOMANDA"      => $nrComanda,
                            "NRPRODCOMVEN"   => $observationDetails["NRPRODCOMVEN"],
                            "CDPRODUTO"      => $observationDetails["CDPRODUTO"],
                            "QTPRODCOMVEN"   => 1.00,
                            "VRPRECCOMVEN"   => $this->util->toFloat($obs['price']),
                            "TXPRODCOMVEN"   => " ",
                            "IDSTPRCOMVEN"   => $idControProd === "S" ? "1" : "3",
                            "VRDESCCOMVEN"   => 0.00,
                            "NRLUGARMESA"    => "01",
                            "IDPRODIMPFIS"   => "N",
                            "CDLOJA"         => $cdLoja,
                            "VRACRCOMVEN"    => 0.00,
                            "NRSEQPRODCOM"   => $this->formatNrSeqProdCom($NRSEQPRODCOM),
                            "NRSEQPRODCUP"   => $orderItemsSeq,
                            "VRPRECCLCOMVEN" => 0.00,
                            "CDCAIXACOLETOR" => $sellerPos,
                            "CDPRODPROMOCAO" => $CDPRODPROMO,
                            "CDVENDEDOR"     => $cdvendedor,
                            "NRINSCRCONS"    => null,
                            "DSOBSPEDDIGCMD" => '',
                            "CDSENHAPED"     => $pagerId,
                            "IDOBSERVACAOAD" => $isCombo ? 'N' : 'S',
                            "NRSEQEXTPROD"   => $obs["id"],
                            "IDPRODEXT"      => $item['id']
                        );

                        array_push($simpleItens, $obsReturn);
                    }

                    if (strlen($obs['code']) === 5 && substr($obs['code'], 2, 1) === '|') {
                        array_push($productReturn['ITEM_OBSERVATION'], [
                            'CDFILIAL'        => $cdFilial,
                            'NRVENDAREST'     => $nrVendarest,
                            'NRCOMANDA'       => $nrComanda,
                            'NRPRODCOMVEN'    => $this->util->toRight($NRPRODCOMVEN, 6),
                            'CDGRPOCOR'       => $observationDetails['CDGRPOCOR'],
                            'CDOCORR'         => $observationDetails['CDOCORR'],
                            'NRPRODCOMVENOBS' => null, //$observationDetails["NRPRODCOMVEN"],  
                            'QTPRODCOMVENOBS' => $obs['quantity']
                        ]);
                    }
                } else {
                    $observationDetails = $this->productFactory->getObservation($obs['code']);

                    if (empty($observationDetails))
                        throw new \Exception("Não foi possivel encontrar o complemento de código: {$obs['code']}");

                    array_push($productReturn['ITEM_OBSERVATION'], [
                        'CDFILIAL'        => $cdFilial,
                        'NRVENDAREST'     => $nrVendarest,
                        'NRCOMANDA'       => $nrComanda,
                        'NRPRODCOMVEN'    => $this->util->toRight($NRPRODCOMVEN, 6),
                        'CDPRODUTO'       => $observationDetails['CDPRODUTO'],
                        'CDGRPOCOR'       => $observationDetails['CDGRPOCOR'],
                        'CDOCORR'         => $observationDetails['CDOCORR'],
                        'NRPRODCOMVENOBS' => null,
                        'QTPRODCOMVENOBS' => 1
                    ]);
                }
            }
        }
        array_push($simpleItens, $productReturn);

        return $simpleItens;
    }

    private function getObservationName($observation)
    {

        if (empty($observation["code"]))
            throw new \Exception("A observação: " . $observation["name"] . ", não possui código de integração parametrizado.");

        $isAcr = false;
        if ($observation['price'] >= 0.01) {
            $isAcr = true;
        }

        $observationDetails = $this->productFactory->getObservation($observation["code"], $isAcr);

        if (empty($observationDetails))
            throw new \Exception("Não foi encontrada ocorrência para o código de integração: " . $observation["code"] . " Complemento - " . $observation["name"] . " .Verifique o cadastro de observações.");

        return $observationDetails["DSOCORR"];
    }

    private function formatNrSeqProdCom($seq)
    {
        return str_pad((string) ((int) $seq), 3, '0', STR_PAD_LEFT);
    }

    public function insertProduct($item, $generalParams)
    {
        return $this->productFactory->insertProduct($item, $generalParams);
    }

    public function insertComandaEst($item, $generalParams)
    {
        $updateProduct = $this->productFactory->getItcomandaEstProduct($item);
        if(empty($updateProduct)) {
            $this->productFactory->insertItComandaEst($item, $generalParams);
        } else {
            $item['QTPROCOMEST'] += $updateProduct['QTPROCOMEST'];
            $this->productFactory->updateItComandaEst($item, $generalParams);
        }
        
    }

    public function insertObservation($item, $generalParams, $orderData)
    {
        return $this->productFactory->insertObservation($item, $generalParams, $orderData);
    }

    public function insertObsItComandaEst($item, $generalParams, $orderData)
    {
        return $this->productFactory->insertObsItComandaEst($item, $generalParams, $orderData);
    }

    public function ProductsForShipping($cdFilial, $produtsPartner)
    {
        $cdTabePrec    = $this->productFactory->getTabePrec($cdFilial);
        $products      = $this->productFactory->ProductsForShipping($cdFilial, $cdTabePrec);
        $observations  = $this->productFactory->observationsForShipping($cdFilial);
        $rodizioProds  = $this->productFactory->rodizioProductsForShipping($cdFilial);
        $products      = array_merge($products, $observations);
        $arrayProducts = [];

        foreach ($products as $product) {
            $isExist = false;

            if (!empty($produtsPartner)) {

                //  VALIDA SE O PRODUTO JÁ FOI CADASTRADO NA ABRAHAO
                foreach ($produtsPartner as $prodExt) {
                    if ($product['CDARVPROD'] == $prodExt['code']) {
                        $isExist = true;
                    }
                }
            }

            if (!$isExist) {
                $arrayProd = [
                    'code'  => $product['CDARVPROD'],
                    'name'  => $product['NMPRODUTO'],
                    'price' => !empty($product['PRECOFILI']) ? $this->util->toFloat($product['PRECOFILI']) : $this->util->toFloat($product['PRECOGER']),
                ];
                array_push($arrayProducts, $arrayProd);
                $contente = date("d:m:Y H:i:s") . " - " . "Cadastrando produto {$product['CDARVPROD']} - {$product['NMPRODUTO']} com preco R$ {$arrayProd['price']}";
                $this->util->writeLog('sendProducts_execution', '', $contente);
            }
        }

        foreach ($rodizioProds as $rodProd) {
            $isExist = false;

            if (!empty($produtsPartner)) {

                //  VALIDA SE O PRODUTO JÁ FOI CADASTRADO NA ABRAHAO
                foreach ($produtsPartner as $prodExt) {
                    if ("RODIZIO@{$rodProd['CDARVPROD']}" == $prodExt['code']) {
                        $isExist = true;
                    }
                }
            }

            if (!$isExist) {
                $arrayRodizioProd = [
                    'code'  => "RODIZIO@{$rodProd['CDARVPROD']}",
                    'name'  => $rodProd['NMPRODUTO'],
                    'price' => 0,
                ];
                array_push($arrayProducts, $arrayRodizioProd);
                $contente = date("d:m:Y H:i:s") . " - " . "Cadastrando produto {$arrayRodizioProd['code']} - {$arrayRodizioProd['name']} com preco R$ {$arrayRodizioProd['price']}";
                $this->util->writeLog('sendProducts_execution', '', $contente);
            }
        }
        return $arrayProducts;
    }

    public function ProductsToUpdating($cdFilial, $produtsPartner)
    {
        $cdTabePrec   = $this->productFactory->getTabePrec($cdFilial);
        $products     = $this->productFactory->ProductsForShipping($cdFilial, $cdTabePrec);
        $observations = $this->productFactory->observationsForShipping($cdFilial);
        $rodizioProds = $this->productFactory->rodizioProductsForShipping($cdFilial);
        $products     = array_merge($products, $observations);
        $update       = [];

        foreach ($products as $product) {
            $isDiff = false;

            if (!empty($produtsPartner)) {

                //  VALIDA SE O VALOR DO PRODUTO É DIFERENTE DO SISTEMA TEKNISA
                foreach ($produtsPartner as $prodExt) {
                    if ($product['CDARVPROD'] == $prodExt['code']) {
                        $precoFili = !empty($product['PRECOFILI']) ? $this->util->toFloat($product['PRECOFILI']) : 0;
                        $precoGer = !empty($product['PRECOGER']) ? $this->util->toFloat($product['PRECOGER']) : 0;
                        $price = !empty($prodExt['price']) ? $this->util->toFloat($prodExt['price']) : 0;

                        if ((empty($precoFili)) && ($precoGer != $price)) {
                            $isDiff = true;
                        } elseif ((!empty($precoFili)) && ($precoFili != $price)) {
                            $isDiff = true;
                        }
                    }
                }
            }

            if (($isDiff)) {
                $prodUpdate = [
                    'code'    => $product['CDARVPROD'],
                    'request'   => [
                        'name'  => $product['NMPRODUTO'],
                        'price' => !empty($product['PRECOFILI']) ? $this->util->toFloat($product['PRECOFILI']) : $this->util->toFloat($product['PRECOGER'])
                    ]
                ];
                array_push($update, $prodUpdate);
            }
        }

        foreach ($rodizioProds as $rodProd) {
            $isDiff = false;

            if (!empty($produtsPartner)) {

                //  VALIDA SE O VALOR DO PRODUTO É DIFERENTE DO SISTEMA TEKNISA
                foreach ($produtsPartner as $prodExt) {
                    if ("RODIZIO@{$rodProd['CDARVPROD']}" == $prodExt['code']) {
                        $price = !empty($prodExt['price']) ? $this->util->toFloat($prodExt['price']) : 0;

                        if ($price != 0) {
                            $isDiff = true;
                        }
                    }
                }
            }

            if (($isDiff)) {
                $rodizioProdUpdate = [
                    'code'    => "RODIZIO@{$rodProd['CDARVPROD']}",
                    'request'   => [
                        'name'  => $rodProd['NMPRODUTO'],
                        'price' => 0
                    ]
                ];
                array_push($update, $rodizioProdUpdate);
            }
        }
        return $update;
    }

    private function processRodizio($item, $orderType, $cdFilial, $nrVendarest, $nrComanda, $cdLoja, $idControProd, $sellerPos, $cdvendedor, $pagerId, $prodRodizio)
    {   
        if(!empty($prodRodizio)) {
            $mainProduct = $prodRodizio;
        } else {
            $mainProduct = $this->productFactory->getMainProduct($item['main_item_id'], $cdFilial, $nrVendarest, $nrComanda);
        }

        if(empty($mainProduct))
        throw new \Exception("Não foi encontrado lançamento de rodizio para relacionamento do produto: {$item['code']} - {$item['name']}.");
        

        $rodizioItens = [];

        if (empty($item['code']))
            throw new \Exception("O produto: {$item['code']} - {$item['name']} não possui código de integração parametrizado.");

        $item['code'] = str_replace("RODIZIO@", "", $item['code']);
        $productDetails = $this->productFactory->getProduct($item['code']);
        if (empty($productDetails))
            throw new \Exception("O produto: {$item['code']} - {$item['name']} não encontrado. Verifique a parametrização.");


        $productDetails = array_merge($productDetails, $item);

        $productDetails["TXPRODCOMVENEST"] = " ";
        $hasComplement = false;

        if (!empty($item["options"]) && count($item["options"]) > 0) {
            foreach ($productDetails["options"] as $obs) {
                if (strlen($obs['code']) === 5 && substr($obs['code'], 2, 1) === '|') {
                    $obsTitle = $this->getObservationName($obs);
                    for ($i = 0; $i < $obs["quantity"]; $i++)
                        $productDetails["TXPRODCOMVENEST"] .= $obsTitle . "; ";
                }
                if (($obs['price'] > 0.01) && (!$hasComplement))
                    $hasComplement = true;
            }
        }
        $CDPRODPROMO = null;
        if ((!empty($item["options"])) && ($hasComplement))
            $CDPRODPROMO = $productDetails["CDPRODUTO"];

        $notes = "";

        if (!empty($item['notes']) && count($item['notes']) > 0) {
            foreach ($item['notes'] as $note) {
                $notes .= $note . "; ";
            }
        }

        $productDetails["TXPRODCOMVENEST"] .= empty($productDetails["notes"]) ? "" : implode(",", $productDetails["notes"]) . "; ";

        $productReturn = array(
            "CDFILIAL"          => $cdFilial,
            "NRVENDAREST"       => $nrVendarest,
            "NRCOMANDA"         => $nrComanda,
            "NRPRODCOMVEN"      => $this->util->toRight($mainProduct['NRPRODCOMVEN'], 6),
            "CDPRODUTO"         => $productDetails["CDPRODUTO"],
            "QTPROCOMEST"      => $item['quantity'],
            "VRPRECCOMEST"      => $this->util->toFloat($item['price']),
            "TXPRODCOMVENEST"   => $productDetails["TXPRODCOMVENEST"],
            "VRDESITCOMEST"     => 0.00,
            "VRACRITCOMEST"     => 0.00,
            "DSOBSPEDDIGEST"    => $notes,
            "IDIMPPRGRPROMO"    => 'N',
            "ITEM_OBSERVATION"  => [],
            "IDPRODEXT"         => $item['id']
        );

        if (!empty($productDetails["options"]) && count($productDetails["options"]) > 0) {
            foreach ($productDetails["options"] as $obs) {
                if ($obs['price'] >= 0.01) {
                    $orderItemsSeq = $this->util->geraCodigo("ITCOMANDAVEN$cdFilial", 1, 10);
                    $NRPRODCOMVEN  = ($this->util->geraCodigo("ITCOMANDAVEN{$cdFilial}{$nrComanda}", 1, 6) - 1);
                    $NRSEQPRODCOM  = $NRPRODCOMVEN;

                    $isCombo = false;

                    if (strlen($obs['code']) === 5 && substr($obs['code'], 2, 1) === '|') {
                        $observationDetails = $this->productFactory->getObservation($obs['code'], true);
                    } else {
                        $isCombo = true;
                        $observationDetails = $this->productFactory->getProduct($obs['code']);
                    }

                    if (empty($observationDetails))
                        throw new \Exception("Não foi possivel encontrar o complemento de código: {$obs['code']}");

                    for ($y = 0; $y < $obs['quantity']; $y++) {
                        $NRPRODCOMVEN = ($this->util->geraCodigo("ITCOMANDAVEN{$cdFilial}{$nrComanda}", 1, 6) - 1);
                        $observationDetails["NRPRODCOMVEN"] = str_pad((string) ($NRPRODCOMVEN), 6, '0', STR_PAD_LEFT);


                        $obsReturn = array(
                            "CDFILIAL"       => $cdFilial,
                            "NRVENDAREST"    => $nrVendarest,
                            "NRCOMANDA"      => $nrComanda,
                            "NRPRODCOMVEN"   => $observationDetails["NRPRODCOMVEN"],
                            "CDPRODUTO"      => $observationDetails["CDPRODUTO"],
                            "QTPRODCOMVEN"   => 1.00,
                            "VRPRECCOMVEN"   => $this->util->toFloat($obs['price']),
                            "TXPRODCOMVEN"   => " ",
                            "IDSTPRCOMVEN"   => $idControProd === "S" ? "1" : "3",
                            "VRDESCCOMVEN"   => 0.00,
                            "NRLUGARMESA"    => "01",
                            "IDPRODIMPFIS"   => "N",
                            "CDLOJA"         => $cdLoja,
                            "VRACRCOMVEN"    => 0.00,
                            "NRSEQPRODCOM"   => $this->formatNrSeqProdCom($NRSEQPRODCOM),
                            "NRSEQPRODCUP"   => $orderItemsSeq,
                            "VRPRECCLCOMVEN" => 0.00,
                            "CDCAIXACOLETOR" => $sellerPos,
                            "CDPRODPROMOCAO" => $CDPRODPROMO,
                            "CDVENDEDOR"     => $cdvendedor,
                            "NRINSCRCONS"    => null,
                            "DSOBSPEDDIGCMD" => '',
                            "CDSENHAPED"     => $pagerId,
                            "IDOBSERVACAOAD" => $isCombo ? 'N' : 'S',
                            "NRSEQEXTPROD"   => $obs["id"],
                            "IDPRODEXT"      => $item['id']
                        );

                        array_push($simpleItens, $obsReturn);
                    }

                    if (strlen($obs['code']) === 5 && substr($obs['code'], 2, 1) === '|') {
                        array_push($productReturn['ITEM_OBSERVATION'], [
                            'CDFILIAL'        => $cdFilial,
                            'NRVENDAREST'     => $nrVendarest,
                            'NRCOMANDA'       => $nrComanda,
                            'NRPRODCOMVEN'    => $this->util->toRight($NRPRODCOMVEN, 6),
                            'CDPRODUTO'       => $observationDetails['CDPRODUTO'],
                            'CDGRPOCOR'       => $observationDetails['CDGRPOCOR'],
                            'CDOCORR'         => $observationDetails['CDOCORR'],
                            'NRPRODCOMVENOBS' => null, //$observationDetails["NRPRODCOMVEN"],  
                            'QTPRODCOMVENOBS' => $obs['quantity']
                        ]);
                    }
                } else {
                    $observationDetails = $this->productFactory->getObservation($obs['code']);

                    if (empty($observationDetails))
                        throw new \Exception("Não foi possivel encontrar o complemento de código: {$obs['code']}");

                    array_push($productReturn['ITEM_OBSERVATION'], [
                        'CDFILIAL'        => $cdFilial,
                        'NRVENDAREST'     => $nrVendarest,
                        'NRCOMANDA'       => $nrComanda,
                        'NRPRODCOMVEN'    => $this->util->toRight($mainProduct['NRPRODCOMVEN'], 6),
                        'CDGRPOCOR'       => $observationDetails['CDGRPOCOR'],
                        'CDPRODUTO'       => $observationDetails['CDPRODUTO'],
                        'CDOCORR'         => $observationDetails['CDOCORR'],
                        'NRPRODCOMVENOBS' => null,
                        'QTPRODCOMVENOBS' => 1
                    ]);
                }
            }
        }
        array_push($rodizioItens, $productReturn);

        return $rodizioItens;
    }
}
